#include <iostream>
#include <cmath>
#include <cstdlib>

#include "PolyAbstr.hpp"

template <class Coeff>
void Polynom<Coeff>::zero() {
  monoms.clear();
};

template <class Coeff>
void Polynom<Coeff>::set_coeff(int d, Coeff co) {
  if (d >= 0) {
    if (co != 0)
      monoms[d] = co;
    else
      monoms.erase(d);
  }
  else {
    std::cerr << "Mauvais degré " << d << " set_coeff impossible !" << std::endl;
  }
}

template <class Coeff>
int Polynom<Coeff>::degree() const {
  int res = -1;
  typename std::map<int,Coeff>::const_iterator it;
  for (it = monoms.begin(); it != monoms.end(); ++it)
    if (res < it->first) res = it->first;
  return res;
}

template <class Coeff>
Coeff Polynom<Coeff>::coeff(int d) const {
  typename std::map<int,Coeff>::const_iterator it = monoms.find(d);
  if (it != monoms.end()) return it->second;
  else return 0.;

}

template <class Coeff>
bool Polynom<Coeff>::is_zero() const {
  return monoms.empty();
};


template <class Coeff>
Coeff Polynom<Coeff>::eval(Coeff x) const {
  int res = 0.;
  typename std::map<int,Coeff>::const_iterator it;
  for (it = monoms.begin(); it != monoms.end(); ++it)
    res += it->second * pow(x, it->first);
  return res;
}

/////////////////////////////////////////////////////////////////////////////////
//   fonctions wrapper pour que les étudiants n'ai pas à toucher de l'objet:   //
/////////////////////////////////////////////////////////////////////////////////

typedef Polynom<float> Polynome;

void  PolynomeNul(Polynome &p) { p.zero(); }
void  ModifierCoeffPoly(Polynome &p, int d, float co) { p.set_coeff(d, co); }
int   DegrePoly(const Polynome &p) { return p.degree(); }
float CoeffPoly(const Polynome &p, int d) { return p.coeff(d); }
bool  EstNulPoly(const Polynome &p) { return p.is_zero(); }
float ValeurPoly(const Polynome &p, float v) { return p.eval(v); }
